@@ -2,18 +2,31 @@
 Contains the ABC bus implementation and its documentation.
 """
 
-from typing import cast, Any, Iterator, List, Optional, Sequence, Tuple, Union
-
-import can.typechecking
-
-from abc import ABC, ABCMeta, abstractmethod
-import can
+import contextlib
 import logging
 import threading
-from time import time
+from abc import ABC, ABCMeta, abstractmethod
 from enum import Enum, auto
+from time import time
+from types import TracebackType
+from typing import (
+    Any,
+    Callable,
+    Iterator,
+    List,
+    Optional,
+    Sequence,
+    Tuple,
+    Type,
+    Union,
+    cast,
+)
 
-from can.broadcastmanager import ThreadBasedCyclicSendTask, CyclicSendTaskABC
+from typing_extensions import Self
+
+import can
+import can.typechecking
+from can.broadcastmanager import CyclicSendTaskABC, ThreadBasedCyclicSendTask
 from can.message import Message
 
 LOG = logging.getLogger(__name__)
@@ -27,6 +40,14 @@ class BusState(Enum):
     ERROR = auto()
 
 
+class CanProtocol(Enum):
+    """The CAN protocol type supported by a :class:`can.BusABC` instance"""
+
+    CAN_20 = auto()
+    CAN_FD = auto()
+    CAN_XL = auto()
+
+
 class BusABC(metaclass=ABCMeta):
     """The CAN Bus Abstract Base Class that serves as the basis
     for all concrete interfaces.
@@ -44,12 +65,16 @@ class BusABC(metaclass=ABCMeta):
     #: Log level for received messages
     RECV_LOGGING_LEVEL = 9
 
+    #: Assume that no cleanup is needed until something was initialized
+    _is_shutdown: bool = True
+    _can_protocol: CanProtocol = CanProtocol.CAN_20
+
     @abstractmethod
     def __init__(
         self,
         channel: Any,
         can_filters: Optional[can.typechecking.CanFilters] = None,
-        **kwargs: object
+        **kwargs: object,
     ):
         """Construct and open a CAN bus instance of the specified type.
 
@@ -73,6 +98,10 @@ class BusABC(metaclass=ABCMeta):
         """
         self._periodic_tasks: List[_SelfRemovingCyclicTask] = []
         self.set_filters(can_filters)
+        # Flip the class default value when the constructor finishes.  That
+        # usually means the derived class constructor was also successful,
+        # since it calls this parent constructor last.
+        self._is_shutdown: bool = False
 
     def __str__(self) -> str:
         return self.channel_info
@@ -93,7 +122,6 @@ class BusABC(metaclass=ABCMeta):
         time_left = timeout
 
         while True:
-
             # try to get a message
             msg, already_filtered = self._recv_internal(timeout=time_left)
 
@@ -109,7 +137,6 @@ class BusABC(metaclass=ABCMeta):
             # try next one only if there still is time, and with
             # reduced timeout
             else:
-
                 time_left = timeout - (time() - start)
 
                 if time_left > 0:
@@ -187,6 +214,7 @@ class BusABC(metaclass=ABCMeta):
         period: float,
         duration: Optional[float] = None,
         store_task: bool = True,
+        modifier_callback: Optional[Callable[[Message], None]] = None,
     ) -> can.broadcastmanager.CyclicSendTaskABC:
         """Start sending messages at a given period on this bus.
 
@@ -208,6 +236,10 @@ class BusABC(metaclass=ABCMeta):
         :param store_task:
             If True (the default) the task will be attached to this Bus instance.
             Disable to instead manage tasks manually.
+        :param modifier_callback:
+            Function which should be used to modify each message's data before
+            sending. The callback modifies the :attr:`~can.Message.data` of the
+            message and returns ``None``.
         :return:
             A started task instance. Note the task can be stopped (and depending on
             the backend modified) by calling the task's
@@ -222,7 +254,7 @@ class BusABC(metaclass=ABCMeta):
 
         .. note::
 
-            For extremely long running Bus instances with many short lived
+            For extremely long-running Bus instances with many short-lived
             tasks the default api with ``store_task==True`` may not be
             appropriate as the stopped tasks are still taking up memory as they
             are associated with the Bus instance.
@@ -239,9 +271,8 @@ class BusABC(metaclass=ABCMeta):
         # Create a backend specific task; will be patched to a _SelfRemovingCyclicTask later
         task = cast(
             _SelfRemovingCyclicTask,
-            self._send_periodic_internal(msgs, period, duration),
+            self._send_periodic_internal(msgs, period, duration, modifier_callback),
         )
-
         # we wrap the task's stop method to also remove it from the Bus's list of tasks
         periodic_tasks = self._periodic_tasks
         original_stop_method = task.stop
@@ -255,7 +286,7 @@ class BusABC(metaclass=ABCMeta):
                     pass  # allow the task to be already removed
             original_stop_method()
 
-        task.stop = wrapped_stop_method  # type: ignore
+        task.stop = wrapped_stop_method  # type: ignore[method-assign]
 
         if store_task:
             self._periodic_tasks.append(task)
@@ -267,6 +298,7 @@ class BusABC(metaclass=ABCMeta):
         msgs: Union[Sequence[Message], Message],
         period: float,
         duration: Optional[float] = None,
+        modifier_callback: Optional[Callable[[Message], None]] = None,
     ) -> can.broadcastmanager.CyclicSendTaskABC:
         """Default implementation of periodic message sending using threading.
 
@@ -290,7 +322,12 @@ class BusABC(metaclass=ABCMeta):
                 threading.Lock()
             )
         task = ThreadBasedCyclicSendTask(
-            self, self._lock_send_periodic, msgs, period, duration
+            bus=self,
+            lock=self._lock_send_periodic,
+            messages=msgs,
+            period=period,
+            duration=duration,
+            modifier_callback=modifier_callback,
         )
         return task
 
@@ -303,6 +340,10 @@ class BusABC(metaclass=ABCMeta):
         :param remove_tasks:
             Stop tracking the stopped tasks.
         """
+        if not hasattr(self, "_periodic_tasks"):
+            # avoid AttributeError for partially initialized BusABC instance
+            return
+
         for task in self._periodic_tasks:
             # we cannot let `task.stop()` modify `self._periodic_tasks` while we are
             # iterating over it (#634)
@@ -314,8 +355,10 @@ class BusABC(metaclass=ABCMeta):
     def __iter__(self) -> Iterator[Message]:
         """Allow iteration on messages as they are received.
 
-            >>> for msg in bus:
-            ...     print(msg)
+        .. code-block:: python
+
+            for msg in bus:
+                print(msg)
 
 
         :yields:
@@ -352,9 +395,9 @@ class BusABC(metaclass=ABCMeta):
 
         :param filters:
             A iterable of dictionaries each containing a "can_id",
-            a "can_mask", and an optional "extended" key.
+            a "can_mask", and an optional "extended" key::
 
-            >>> [{"can_id": 0x11, "can_mask": 0x21, "extended": False}]
+                [{"can_id": 0x11, "can_mask": 0x21, "extended": False}]
 
             A filter matches, when
             ``<received_can_id> & can_mask == can_id & can_mask``.
@@ -363,7 +406,8 @@ class BusABC(metaclass=ABCMeta):
             messages based only on the arbitration ID and mask.
         """
         self._filters = filters or None
-        self._apply_filters(self._filters)
+        with contextlib.suppress(NotImplementedError):
+            self._apply_filters(self._filters)
 
     def _apply_filters(self, filters: Optional[can.typechecking.CanFilters]) -> None:
         """
@@ -373,6 +417,7 @@ class BusABC(metaclass=ABCMeta):
         :param filters:
             See :meth:`~can.BusABC.set_filters` for details.
         """
+        raise NotImplementedError
 
     def _matches_filters(self, msg: Message) -> bool:
         """Checks whether the given message matches at least one of the
@@ -412,20 +457,40 @@ class BusABC(metaclass=ABCMeta):
 
     def flush_tx_buffer(self) -> None:
         """Discard every message that may be queued in the output buffer(s)."""
+        raise NotImplementedError
 
     def shutdown(self) -> None:
         """
-        Called to carry out any interface specific cleanup required
-        in shutting down a bus.
+        Called to carry out any interface specific cleanup required in shutting down a bus.
+
+        This method can be safely called multiple times.
         """
+        if self._is_shutdown:
+            LOG.debug("%s is already shut down", self.__class__)
+            return
+
+        self._is_shutdown = True
         self.stop_all_periodic_tasks()
 
-    def __enter__(self):
+    def __enter__(self) -> Self:
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(
+        self,
+        exc_type: Optional[Type[BaseException]],
+        exc_value: Optional[BaseException],
+        traceback: Optional[TracebackType],
+    ) -> None:
         self.shutdown()
 
+    def __del__(self) -> None:
+        if not self._is_shutdown:
+            LOG.warning("%s was not properly shut down", self.__class__.__name__)
+            # We do some best-effort cleanup if the user
+            # forgot to properly close the bus instance
+            with contextlib.suppress(AttributeError):
+                self.shutdown()
+
     @property
     def state(self) -> BusState:
         """
@@ -440,6 +505,15 @@ class BusABC(metaclass=ABCMeta):
         """
         raise NotImplementedError("Property is not implemented.")
 
+    @property
+    def protocol(self) -> CanProtocol:
+        """Return the CAN protocol used by this bus instance.
+
+        This value is set at initialization time and does not change
+        during the lifetime of a bus instance.
+        """
+        return self._can_protocol
+
     @staticmethod
     def _detect_available_configs() -> List[can.typechecking.AutoDetectedConfig]:
         """Detect all configurations/channels that this interface could
@@ -464,7 +538,5 @@ class _SelfRemovingCyclicTask(CyclicSendTaskABC, ABC):
     Only needed for typing :meth:`Bus._periodic_tasks`. Do not instantiate.
     """
 
-    def stop(  # pylint: disable=arguments-differ
-        self, remove_task: bool = True
-    ) -> None:
+    def stop(self, remove_task: bool = True) -> None:
         raise NotImplementedError()
