@@ -14,7 +14,19 @@ import socket
 import struct
 import threading
 import time
-from typing import Dict, List, Optional, Sequence, Tuple, Type, Union
+import warnings
+from typing import Callable, Dict, List, Optional, Sequence, Tuple, Type, Union
+
+import can
+from can import BusABC, CanProtocol, Message
+from can.broadcastmanager import (
+    LimitedDurationCyclicSendTaskABC,
+    ModifiableCyclicTaskABC,
+    RestartableCyclicTaskABC,
+)
+from can.interfaces.socketcan import constants
+from can.interfaces.socketcan.utils import find_available_interfaces, pack_filters
+from can.typechecking import CanFilters
 
 log = logging.getLogger(__name__)
 log_tx = log.getChild("tx")
@@ -29,18 +41,6 @@ except ImportError:
     log.error("socket.CMSG_SPACE not available on this platform")
 
 
-import can
-from can import BusABC, Message
-from can.broadcastmanager import (
-    LimitedDurationCyclicSendTaskABC,
-    ModifiableCyclicTaskABC,
-    RestartableCyclicTaskABC,
-)
-from can.interfaces.socketcan import constants
-from can.interfaces.socketcan.utils import find_available_interfaces, pack_filters
-from can.typechecking import CanFilters
-
-
 # Setup BCM struct
 def bcm_header_factory(
     fields: List[Tuple[str, Union[Type[ctypes.c_uint32], Type[ctypes.c_long]]]],
@@ -536,7 +536,9 @@ def capture_message(
         else:
             channel = None
     except OSError as error:
-        raise can.CanOperationError(f"Error receiving: {error.strerror}", error.errno)
+        raise can.CanOperationError(
+            f"Error receiving: {error.strerror}", error.errno
+        ) from error
 
     can_id, can_dlc, flags, data = dissect_can_frame(cf)
 
@@ -601,7 +603,7 @@ if CMSG_SPACE_available:
     RECEIVED_ANCILLARY_BUFFER_SIZE = CMSG_SPACE(RECEIVED_TIMESTAMP_STRUCT.size)
 
 
-class SocketcanBus(BusABC):
+class SocketcanBus(BusABC):  # pylint: disable=abstract-method
     """A SocketCAN interface to CAN.
 
     It implements :meth:`can.BusABC._detect_available_configs` to search for
@@ -656,6 +658,7 @@ class SocketcanBus(BusABC):
         self._is_filtered = False
         self._task_id = 0
         self._task_id_guard = threading.Lock()
+        self._can_protocol = CanProtocol.CAN_FD if fd else CanProtocol.CAN_20
 
         # set the local_loopback parameter
         try:
@@ -710,7 +713,11 @@ class SocketcanBus(BusABC):
                 "local_loopback": local_loopback,
             }
         )
-        super().__init__(channel=channel, can_filters=can_filters, **kwargs)
+        super().__init__(
+            channel=channel,
+            can_filters=can_filters,
+            **kwargs,
+        )
 
     def shutdown(self) -> None:
         """Stops all active periodic tasks and closes the socket."""
@@ -732,7 +739,7 @@ class SocketcanBus(BusABC):
             # something bad happened (e.g. the interface went down)
             raise can.CanOperationError(
                 f"Failed to receive: {error.strerror}", error.errno
-            )
+            ) from error
 
         if ready_receive_sockets:  # not empty
             get_channel = self.channel == ""
@@ -793,7 +800,7 @@ class SocketcanBus(BusABC):
         except OSError as error:
             raise can.CanOperationError(
                 f"Failed to transmit: {error.strerror}", error.errno
-            )
+            ) from error
         return sent
 
     def _send_periodic_internal(
@@ -801,7 +808,8 @@ class SocketcanBus(BusABC):
         msgs: Union[Sequence[Message], Message],
         period: float,
         duration: Optional[float] = None,
-    ) -> CyclicSendTask:
+        modifier_callback: Optional[Callable[[Message], None]] = None,
+    ) -> can.broadcastmanager.CyclicSendTaskABC:
         """Start sending messages at a given period on this bus.
 
         The Linux kernel's Broadcast Manager SocketCAN API is used to schedule
@@ -833,15 +841,30 @@ class SocketcanBus(BusABC):
             general the message will be sent at the given rate until at
             least *duration* seconds.
         """
-        msgs = LimitedDurationCyclicSendTaskABC._check_and_convert_messages(  # pylint: disable=protected-access
-            msgs
-        )
+        if modifier_callback is None:
+            msgs = LimitedDurationCyclicSendTaskABC._check_and_convert_messages(  # pylint: disable=protected-access
+                msgs
+            )
 
-        msgs_channel = str(msgs[0].channel) if msgs[0].channel else None
-        bcm_socket = self._get_bcm_socket(msgs_channel or self.channel)
-        task_id = self._get_next_task_id()
-        task = CyclicSendTask(bcm_socket, task_id, msgs, period, duration)
-        return task
+            msgs_channel = str(msgs[0].channel) if msgs[0].channel else None
+            bcm_socket = self._get_bcm_socket(msgs_channel or self.channel)
+            task_id = self._get_next_task_id()
+            task = CyclicSendTask(bcm_socket, task_id, msgs, period, duration)
+            return task
+
+        # fallback to thread based cyclic task
+        warnings.warn(
+            f"{self.__class__.__name__} falls back to a thread-based cyclic task, "
+            "when the `modifier_callback` argument is given.",
+            stacklevel=3,
+        )
+        return BusABC._send_periodic_internal(
+            self,
+            msgs=msgs,
+            period=period,
+            duration=duration,
+            modifier_callback=modifier_callback,
+        )
 
     def _get_next_task_id(self) -> int:
         with self._task_id_guard:
@@ -877,35 +900,3 @@ class SocketcanBus(BusABC):
             {"interface": "socketcan", "channel": channel}
             for channel in find_available_interfaces()
         ]
-
-
-if __name__ == "__main__":
-    # This example demonstrates how to use the internal methods of this module.
-    # It creates two sockets on vcan0 to test sending and receiving.
-    #
-    # If you want to try it out you can do the following (possibly using sudo):
-    #
-    #     modprobe vcan
-    #     ip link add dev vcan0 type vcan
-    #     ip link set vcan0 up
-    #
-    log.setLevel(logging.DEBUG)
-
-    def receiver(event: threading.Event) -> None:
-        receiver_socket = create_socket()
-        bind_socket(receiver_socket, "vcan0")
-        print("Receiver is waiting for a message...")
-        event.set()
-        print(f"Receiver got: {capture_message(receiver_socket)}")
-
-    def sender(event: threading.Event) -> None:
-        event.wait()
-        sender_socket = create_socket()
-        bind_socket(sender_socket, "vcan0")
-        msg = Message(arbitration_id=0x01, data=b"\x01\x02\x03")
-        sender_socket.send(build_can_frame(msg))
-        print("Sender sent a message.")
-
-    e = threading.Event()
-    threading.Thread(target=receiver, args=(e,)).start()
-    threading.Thread(target=sender, args=(e,)).start()
