@@ -2,40 +2,123 @@
 See the :class:`Logger` class.
 """
 
+import gzip
 import os
 import pathlib
 from abc import ABC, abstractmethod
 from datetime import datetime
-import gzip
-from typing import Any, Optional, Callable, Type, Tuple, cast, Dict
-
 from types import TracebackType
+from typing import (
+    Any,
+    Callable,
+    ClassVar,
+    Dict,
+    Final,
+    Literal,
+    Optional,
+    Set,
+    Tuple,
+    Type,
+    cast,
+)
 
-from typing_extensions import Literal
-from pkg_resources import iter_entry_points
+from typing_extensions import Self
 
+from .._entry_points import read_entry_points
 from ..message import Message
-from ..listener import Listener
-from .generic import BaseIOHandler, FileIOMessageWriter, MessageWriter
+from ..typechecking import AcceptedIOType, FileLike, StringPathLike
 from .asc import ASCWriter
 from .blf import BLFWriter
 from .canutils import CanutilsLogWriter
 from .csv import CSVWriter
-from .sqlite import SqliteWriter
+from .generic import (
+    BinaryIOMessageWriter,
+    FileIOMessageWriter,
+    MessageWriter,
+)
+from .mf4 import MF4Writer
 from .printer import Printer
-from ..typechecking import StringPathLike, FileLike, AcceptedIOType
+from .sqlite import SqliteWriter
+from .trc import TRCWriter
+
+#: A map of file suffixes to their corresponding
+#: :class:`can.io.generic.MessageWriter` class
+MESSAGE_WRITERS: Final[Dict[str, Type[MessageWriter]]] = {
+    ".asc": ASCWriter,
+    ".blf": BLFWriter,
+    ".csv": CSVWriter,
+    ".db": SqliteWriter,
+    ".log": CanutilsLogWriter,
+    ".mf4": MF4Writer,
+    ".trc": TRCWriter,
+    ".txt": Printer,
+}
 
 
-class Logger(MessageWriter):  # pylint: disable=abstract-method
+def _update_writer_plugins() -> None:
+    """Update available message writer plugins from entry points."""
+    for entry_point in read_entry_points("can.io.message_writer"):
+        if entry_point.key in MESSAGE_WRITERS:
+            continue
+
+        writer_class = entry_point.load()
+        if issubclass(writer_class, MessageWriter):
+            MESSAGE_WRITERS[entry_point.key] = writer_class
+
+
+def _get_logger_for_suffix(suffix: str) -> Type[MessageWriter]:
+    try:
+        return MESSAGE_WRITERS[suffix]
+    except KeyError:
+        raise ValueError(
+            f'No write support for unknown log format "{suffix}"'
+        ) from None
+
+
+def _compress(
+    filename: StringPathLike, **kwargs: Any
+) -> Tuple[Type[MessageWriter], FileLike]:
     """
-    Logs CAN messages to a file.
+    Return the suffix and io object of the decompressed file.
+    File will automatically recompress upon close.
+    """
+    suffixes = pathlib.Path(filename).suffixes
+    if len(suffixes) != 2:
+        raise ValueError(
+            f"No write support for unknown log format \"{''.join(suffixes)}\""
+        ) from None
+
+    real_suffix = suffixes[-2].lower()
+    if real_suffix in (".blf", ".db"):
+        raise ValueError(
+            f"The file type {real_suffix} is currently incompatible with gzip."
+        )
+    logger_type = _get_logger_for_suffix(real_suffix)
+    append = kwargs.get("append", False)
+
+    if issubclass(logger_type, BinaryIOMessageWriter):
+        mode = "ab" if append else "wb"
+    else:
+        mode = "at" if append else "wt"
+
+    return logger_type, gzip.open(filename, mode)
+
+
+def Logger(  # noqa: N802
+    filename: Optional[StringPathLike], **kwargs: Any
+) -> MessageWriter:
+    """Find and return the appropriate :class:`~can.io.generic.MessageWriter` instance
+    for a given file suffix.
 
     The format is determined from the file suffix which can be one of:
-      * .asc: :class:`can.ASCWriter`
+      * .asc :class:`can.ASCWriter`
       * .blf :class:`can.BLFWriter`
       * .csv: :class:`can.CSVWriter`
-      * .db: :class:`can.SqliteWriter`
+      * .db :class:`can.SqliteWriter`
       * .log :class:`can.CanutilsLogWriter`
+      * .mf4 :class:`can.MF4Writer`
+        (optional, depends on `asammdf <https://github.com/danielhrisca/asammdf>`_)
+      * .trc :class:`can.TRCWriter`
       * .txt :class:`can.Printer`
 
     Any of these formats can be used with gzip compression by appending
@@ -46,116 +129,73 @@ class Logger(MessageWriter):  # pylint: disable=abstract-method
 
     The log files may be incomplete until `stop()` is called due to buffering.
 
+    :param filename:
+        the filename/path of the file to write to,
+        may be a path-like object or None to
+        instantiate a :class:`~can.Printer`
+    :raises ValueError:
+        if the filename's suffix is of an unknown file type
+
     .. note::
-        This class itself is just a dispatcher, and any positional and keyword
+        This function itself is just a dispatcher, and any positional and keyword
         arguments are passed on to the returned instance.
     """
 
-    fetched_plugins = False
-    message_writers: Dict[str, Type[MessageWriter]] = {
-        ".asc": ASCWriter,
-        ".blf": BLFWriter,
-        ".csv": CSVWriter,
-        ".db": SqliteWriter,
-        ".log": CanutilsLogWriter,
-        ".txt": Printer,
-    }
+    if filename is None:
+        return Printer(**kwargs)
 
-    @staticmethod
-    def __new__(  # type: ignore
-        cls: Any, filename: Optional[StringPathLike], *args: Any, **kwargs: Any
-    ) -> MessageWriter:
-        """
-        :param filename: the filename/path of the file to write to,
-                         may be a path-like object or None to
-                         instantiate a :class:`~can.Printer`
-        :raises ValueError: if the filename's suffix is of an unknown file type
-        """
-        if filename is None:
-            return Printer(*args, **kwargs)
+    _update_writer_plugins()
 
-        if not Logger.fetched_plugins:
-            Logger.message_writers.update(
-                {
-                    writer.name: writer.load()
-                    for writer in iter_entry_points("can.io.message_writer")
-                }
-            )
-            Logger.fetched_plugins = True
-
-        suffix = pathlib.PurePath(filename).suffix.lower()
-
-        file_or_filename: AcceptedIOType = filename
-        if suffix == ".gz":
-            suffix, file_or_filename = Logger.compress(filename)
-
-        try:
-            return Logger.message_writers[suffix](file_or_filename, *args, **kwargs)
-        except KeyError:
-            raise ValueError(
-                f'No write support for this unknown log format "{suffix}"'
-            ) from None
-
-    @staticmethod
-    def compress(filename: StringPathLike) -> Tuple[str, FileLike]:
-        """
-        Return the suffix and io object of the decompressed file.
-        File will automatically recompress upon close.
-        """
-        real_suffix = pathlib.Path(filename).suffixes[-2].lower()
-        mode = "ab" if real_suffix == ".blf" else "at"
-
-        return real_suffix, gzip.open(filename, mode)
-
-    def on_message_received(self, msg: Message) -> None:
-        pass
+    suffix = pathlib.PurePath(filename).suffix.lower()
+    file_or_filename: AcceptedIOType = filename
+    if suffix == ".gz":
+        logger_type, file_or_filename = _compress(filename, **kwargs)
+    else:
+        logger_type = _get_logger_for_suffix(suffix)
+    return logger_type(file=file_or_filename, **kwargs)
 
 
-class BaseRotatingLogger(Listener, BaseIOHandler, ABC):
+class BaseRotatingLogger(MessageWriter, ABC):
     """
     Base class for rotating CAN loggers. This class is not meant to be
-    instantiated directly. Subclasses must implement the :attr:`should_rollover`
-    and `do_rollover` methods according to their rotation strategy.
+    instantiated directly. Subclasses must implement the :meth:`should_rollover`
+    and :meth:`do_rollover` methods according to their rotation strategy.
 
     The rotation behavior can be further customized by the user by setting
     the :attr:`namer` and :attr:`rotator` attributes after instantiating the subclass.
 
-    These attributes as well as the methods `rotation_filename` and `rotate`
+    These attributes as well as the methods :meth:`rotation_filename` and :meth:`rotate`
     and the corresponding docstrings are carried over from the python builtin
-    `BaseRotatingHandler`.
+    :class:`~logging.handlers.BaseRotatingHandler`.
 
     Subclasses must set the `_writer` attribute upon initialization.
-
-    :attr namer:
-        If this attribute is set to a callable, the :meth:`rotation_filename` method
-        delegates to this callable. The parameters passed to the callable are
-        those passed to :meth:`rotation_filename`.
-    :attr rotator:
-        If this attribute is set to a callable, the :meth:`rotate` method delegates
-        to this callable. The parameters passed to the callable are those
-        passed to :meth:`rotate`.
-    :attr rollover_count:
-        An integer counter to track the number of rollovers.
     """
 
+    _supported_formats: ClassVar[Set[str]] = set()
+
+    #: If this attribute is set to a callable, the :meth:`~BaseRotatingLogger.rotation_filename`
+    #: method delegates to this callable. The parameters passed to the callable are
+    #: those passed to :meth:`~BaseRotatingLogger.rotation_filename`.
     namer: Optional[Callable[[StringPathLike], StringPathLike]] = None
+
+    #: If this attribute is set to a callable, the :meth:`~BaseRotatingLogger.rotate` method
+    #: delegates to this callable. The parameters passed to the callable are those
+    #: passed to :meth:`~BaseRotatingLogger.rotate`.
     rotator: Optional[Callable[[StringPathLike, StringPathLike], None]] = None
+
+    #: An integer counter to track the number of rollovers.
     rollover_count: int = 0
 
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
-        Listener.__init__(self)
-        BaseIOHandler.__init__(self, None)
+    def __init__(self, **kwargs: Any) -> None:
+        super().__init__(**{**kwargs, "file": None})
 
-        self.writer_args = args
         self.writer_kwargs = kwargs
 
-        # Expected to be set by the subclass
-        self._writer: FileIOMessageWriter = None  # type: ignore
-
     @property
+    @abstractmethod
     def writer(self) -> FileIOMessageWriter:
         """This attribute holds an instance of a writer class which manages the actual file IO."""
-        return self._writer
+        raise NotImplementedError
 
     def rotation_filename(self, default_name: StringPathLike) -> StringPathLike:
         """Modify the filename of a log file when rotating.
@@ -163,7 +203,7 @@ class BaseRotatingLogger(Listener, BaseIOHandler, ABC):
         This is provided so that a custom filename can be provided.
         The default implementation calls the :attr:`namer` attribute of the
         handler, if it's callable, passing the default name to
-        it. If the attribute isn't callable (the default is `None`), the name
+        it. If the attribute isn't callable (the default is :obj:`None`), the name
         is returned unchanged.
 
         :param default_name:
@@ -172,14 +212,14 @@ class BaseRotatingLogger(Listener, BaseIOHandler, ABC):
         if not callable(self.namer):
             return default_name
 
-        return self.namer(default_name)
+        return self.namer(default_name)  # pylint: disable=not-callable
 
     def rotate(self, source: StringPathLike, dest: StringPathLike) -> None:
         """When rotating, rotate the current log.
 
         The default implementation calls the :attr:`rotator` attribute of the
-        handler, if it's callable, passing the source and dest arguments to
-        it. If the attribute isn't callable (the default is `None`), the source
+        handler, if it's callable, passing the `source` and `dest` arguments to
+        it. If the attribute isn't callable (the default is :obj:`None`), the source
         is simply renamed to the destination.
 
         :param source:
@@ -193,7 +233,7 @@ class BaseRotatingLogger(Listener, BaseIOHandler, ABC):
             if os.path.exists(source):
                 os.rename(source, dest)
         else:
-            self.rotator(source, dest)
+            self.rotator(source, dest)  # pylint: disable=not-callable
 
     def on_message_received(self, msg: Message) -> None:
         """This method is called to handle the given message.
@@ -208,7 +248,10 @@ class BaseRotatingLogger(Listener, BaseIOHandler, ABC):
         self.writer.on_message_received(msg)
 
     def _get_new_writer(self, filename: StringPathLike) -> FileIOMessageWriter:
-        """Instantiate a new writer after stopping the old one.
+        """Instantiate a new writer.
+
+        .. note::
+            The :attr:`self.writer` should be closed prior to calling this function.
 
         :param filename:
             Path-like object that specifies the location and name of the log file.
@@ -216,20 +259,21 @@ class BaseRotatingLogger(Listener, BaseIOHandler, ABC):
         :return:
             An instance of a writer class.
         """
-        # Close the old writer first
-        if self._writer is not None:
-            self._writer.stop()
+        suffix = "".join(pathlib.Path(filename).suffixes[-2:]).lower()
 
-        logger = Logger(filename, *self.writer_args, **self.writer_kwargs)
-        if isinstance(logger, FileIOMessageWriter):
-            return logger
-        elif isinstance(logger, Printer) and logger.file is not None:
-            return cast(FileIOMessageWriter, logger)
-        else:
-            raise Exception(
-                "The Logger corresponding to the arguments is not a FileIOMessageWriter or "
-                "can.Printer"
-            )
+        if suffix in self._supported_formats:
+            logger = Logger(filename=filename, **self.writer_kwargs)
+            if isinstance(logger, FileIOMessageWriter):
+                return logger
+            elif isinstance(logger, Printer) and logger.file is not None:
+                return cast(FileIOMessageWriter, logger)
+
+        raise ValueError(
+            f'The log format "{suffix}" '
+            f"is not supported by {self.__class__.__name__}. "
+            f"{self.__class__.__name__} supports the following formats: "
+            f"{', '.join(self._supported_formats)}"
+        )
 
     def stop(self) -> None:
         """Stop handling new messages.
@@ -239,7 +283,7 @@ class BaseRotatingLogger(Listener, BaseIOHandler, ABC):
         """
         self.writer.stop()
 
-    def __enter__(self) -> "BaseRotatingLogger":
+    def __enter__(self) -> Self:
         return self
 
     def __exit__(
@@ -248,7 +292,7 @@ class BaseRotatingLogger(Listener, BaseIOHandler, ABC):
         exc_val: Optional[BaseException],
         exc_tb: Optional[TracebackType],
     ) -> Literal[False]:
-        return self._writer.__exit__(exc_type, exc_val, exc_tb)
+        return self.writer.__exit__(exc_type, exc_val, exc_tb)
 
     @abstractmethod
     def should_rollover(self, msg: Message) -> bool:
@@ -267,8 +311,10 @@ class SizedRotatingLogger(BaseRotatingLogger):
     by adding a timestamp and the rollover count. A new log file is then
     created and written to.
 
-    This behavior can be customized by setting the :attr:`namer` and
-    :attr:`rotator` attribute.
+    This behavior can be customized by setting the
+    :attr:`~can.io.BaseRotatingLogger.namer` and
+    :attr:`~can.io.BaseRotatingLogger.rotator`
+    attribute.
 
     Example::
 
@@ -299,10 +345,11 @@ class SizedRotatingLogger(BaseRotatingLogger):
     :meth:`~can.Listener.stop` is called.
     """
 
+    _supported_formats: ClassVar[Set[str]] = {".asc", ".blf", ".csv", ".log", ".txt"}
+
     def __init__(
         self,
         base_filename: StringPathLike,
-        *args: Any,
         max_bytes: int = 0,
         **kwargs: Any,
     ) -> None:
@@ -314,18 +361,22 @@ class SizedRotatingLogger(BaseRotatingLogger):
             The size threshold at which a new log file shall be created. If set to 0, no
             rollover will be performed.
         """
-        super().__init__(*args, **kwargs)
+        super().__init__(**kwargs)
 
         self.base_filename = os.path.abspath(base_filename)
         self.max_bytes = max_bytes
 
         self._writer = self._get_new_writer(self.base_filename)
 
+    @property
+    def writer(self) -> FileIOMessageWriter:
+        return self._writer
+
     def should_rollover(self, msg: Message) -> bool:
         if self.max_bytes <= 0:
             return False
 
-        if self.writer.file.tell() >= self.max_bytes:
+        if self.writer.file_size() >= self.max_bytes:
             return True
 
         return False
@@ -344,11 +395,11 @@ class SizedRotatingLogger(BaseRotatingLogger):
         """Generate the default rotation filename."""
         path = pathlib.Path(self.base_filename)
         new_name = (
-            path.stem
+            path.stem.split(".")[0]
             + "_"
             + datetime.now().strftime("%Y-%m-%dT%H%M%S")
             + "_"
             + f"#{self.rollover_count:03}"
-            + path.suffix
+            + "".join(path.suffixes[-2:])
         )
         return str(path.parent / new_name)
